<?php
/* Copyright (c) 2024 Rapideo. All rights reserved. */
class SynoFileHostingRapideo {
    private $url;
    private $username;
    private $password;
    private $hostinfo;

    private $authtoken_file = "/tmp/rapideo_authtoken.txt";
    private $log_file = '/tmp/rapideo_logfile.txt';


    private $api_url = 'https://www.rapideo.pl/api/rest';
    private $login_endpoint = 'login';
    private $check_file_endpoint = 'files/check';
    private $download_file_endpoint = 'files/download';
	private $log_endpoint = 'log';

    public function __construct($url, $username, $password, $hostinfo) {
        $this->url = $url;
        $this->username = $username;
        $this->password = $password;
        $this->hostinfo = $hostinfo;
    }

    private function api_request($endpoint = "", $method = 'GET' , $data = array(), $conv_json = false) {

        if($conv_json) {
            $json_data = json_encode($data, JSON_PRETTY_PRINT);
            unset($data);
            $data = array($json_data);
        }

        try {
            $curl = curl_init($this->api_url .'/' . $endpoint);
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_TIMEOUT, 10);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER , true);
            if(strtolower($method) == 'post') {
                curl_setopt($curl, CURLOPT_POST, true);
                if(!empty($data)) {
                    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
                }
            }
            curl_setopt($curl, CURLOPT_HTTPHEADER, array('User-Agent: Synology'));
            $response = curl_exec($curl);
			curl_close($curl);

            if($response) {
                $response = json_decode( $response , true);
                if(!empty($response['error'])) {
                    $this->save_log($response['message'] , $response['error']);
                    $this->reset_authtoken();
                }
            }
        } catch(Exception $e) {
            $this->save_log($e->getMessage());
            $this->send_log($e->getMessage());
        }

        return array('response'=>$response);

    }

    private function save_log($message = "" , $error = null) {
        if(is_array($message)) return;

        $date = date("Y-m-d H:i:s" , time());

        try {
            clearstatcache();
            if(file_exists($this->log_file) && filesize($this->log_file) > (1*1024*1024)) {
                unlink($this->log_file);
            }

            $log['message'] = $message;
            if(!empty($error)) $log['message_type'] = 'error';
            else $log['message_type'] = 'notice';

            $log['logtime'] = $date;


            $handle = @fopen($this->log_file , 'a');
            @fwrite($handle,  json_encode($log));
            @fwrite($handle, "\r\n\r\n");
            @fclose($handle);
        } catch(Exception $e) {
            $fp = @fopen($this->log_file , 'a');
            @fwrite($fp, json_encode(array('message'=>$e->getMessage() , 'logtime'=>$date)));
            @fwrite($fp, "\r\n\r\n");
            fclose($fp);
        }
    }

    private function send_log($log = "") {
        $this->api_request($this->log_endpoint , 'post', array('log'=>$log, 'authtoken'=>$this->load_authtoken())  , true);
    }

    //This function returns download url.
    public function GetDownloadInfo() {
        $this->Verify();

        $DownloadInfo = array();
        $check_file = $this->api_request($this->check_file_endpoint, 'post', array('authtoken'=>$this->load_authtoken(), 'url'=>$this->url));

        if(!empty($check_file['response']['file']['filename']) && !empty($check_file['response']['file']['filesize']) && !empty($check_file['response']['file']['hash'])) {
            $get_download_url = $this->api_request($this->download_file_endpoint, 'post' , array('authtoken'=>$this->load_authtoken(),  'hash'=>$check_file['response']['file']['hash'] , 'mode'=>'synology'));
            if(!empty($get_download_url['response']['file']['url'])) {
                $this->save_log('Rozpoczynam pobieranie pliku ' . $check_file['response']['file']['filename_full']);
                $DownloadInfo[INFO_NAME] = $this->hostinfo[INFO_NAME];
                $DownloadInfo[DOWNLOAD_URL] = $get_download_url['response']['file']['url'];
                $DownloadInfo[DOWNLOAD_FILENAME] = $check_file['response']['file']['filename_full'];
                $DownloadInfo[DOWNLOAD_ISPARALLELDOWNLOAD] = true;
            } else {
                $DownloadInfo[DOWNLOAD_ERROR] = $this->getStatus($get_download_url['response']['file']['error'] , $get_download_url['response']['file']['message']);
                $this->save_log($get_download_url['response']['file']['message'] , $get_download_url['response']['file']['error']);
                $this->send_log($get_download_url);
            }
        } else {
            $DownloadInfo[DOWNLOAD_ERROR] = $this->getStatus($check_file['response']['file']['error'], $check_file['response']['file']['message']);
            $this->save_log($check_file['response']['file']['message'], $check_file['response']['file']['error']);
            $this->send_log($check_file);
        }
        return $DownloadInfo;
    }

    private function getStatus($code = 0 , $message = "") {
        switch($code) {
            case "2" :
                return ERR_FILE_NO_EXIST;
                break;
            case "1" :
                return ERR_REQUIRED_PREMIUM;
                break;
            case "9" :
                return ERR_REQUIRED_PREMIUM;
                break;
            case "16" :
                return ERR_NOT_SUPPORT_TYPE;
                break;
            case "3000" :
                return ERR_NOT_SUPPORT_TYPE;
                break;
            default:
                return false;
                break;
        }
    }

    private function reset_authtoken() {
        if(file_exists($this->authtoken_file)) {
            unlink($this->authtoken_file);
        }
        return true;
    }

    private function load_authtoken() {
        if(!file_exists($this->authtoken_file)) return;
        if(filesize($this->authtoken_file) == 0) return;

        $handle = fopen($this->authtoken_file , 'r');
        $authtoken = fread($handle, filesize($this->authtoken_file));
        fclose($handle);
        if(!empty($authtoken)) return $authtoken;
    }

    private function save_authtoken( $authtoken = "") {
        $handle = fopen($this->authtoken_file , 'w');
        fwrite($handle, $authtoken);
        fclose($handle);
    }

    //This function verifies and returns account type.
    public function Verify()
    {
        if(!empty($this->load_authtoken())) return USER_IS_PREMIUM;

        if(empty($this->username) || empty($this->password)) {
            $this->save_log('Wprowadź nazwę użytkownika i hasło w ustawieniach wtyczki' , true);
            return LOGIN_FAIL;
        }

        $return = $this->api_request(
            $this->login_endpoint ,
            'POST',
            array(
                'login' => $this->username ,
                'password'=>$this->password
            )
        );

        if(!empty($return['response']['logged']) && !empty($return['response']['authtoken'])) {
            $this->save_log('Zalogowano w serwisie');
            $this->save_authtoken($return['response']['authtoken']);
            return USER_IS_PREMIUM;
        }

        $this->reset_authtoken();
        if(!empty($return['response']['error'])) {
            switch($return['response']['error']) {
                case "2" :
                    return USER_IS_FREE;
                    break;
            }
        }

        return LOGIN_FAIL;
    }
}
?>
